#include <SPI.h>
#include "bitmapdata.h"

// Declaration for SSD1306 display connected using software SPI (default case):
#define OLED_CS     7   // Slave Select(Chip select)
#define OLED_DC     6   // SSD1306 Data/Command
#define OLED_RESET  5   // SSD1306 Reset

// --- SSD1306 COMMADN : I2Cで使ったのと同じ ---
static uint8_t oledInitCommands[] = { // SSD1306の初期化コマンド群
  0xC8,         // 上下の描画方向
  0xA1,         // 左右の描画方向
  0xA8,0x3F,    // 画面の解像度
  0xD3,0x00,    // 縦方向のオフセット
  0x40,         // 縦方向の描画開始位置
  0xA6,         // 表示方法 - A7にすると画面反転
  0x8D, 0x14    // OLEDパネルの供給電圧と思われる 14hに設定しないとダメらしい
};

uint8_t oledWidth = 128;  // OLEDの幅(ピクセル数)
uint8_t oledHeight = 64;  // OLEDの高さ(ピクセル数)
uint8_t oledPages = 8;    // OLEDのページ数(ピクセル数÷８)
uint8_t SPIBuf[128] ;     // SPI転送用バッファ

// ===== SPI WRITE =====
// OLEDに1バイトコマンドを送信
void oledSendCommand(uint8_t command) {
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

// OLEDに2バイトコマンドを送信
void oledSendCommand2(uint8_t command1,uint8_t command2) {
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

// OLEDに3バイトコマンドを送信
void oledSendCommand3(uint8_t command1,uint8_t command2,uint8_t command3) {
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
  SPI.transfer(command3);
  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

// OLEDに複数のコマンドを送信
void oledSendCommands(int commandLen,uint8_t commands[]) {
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(commands,commandLen);
  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

// 画面を特定のビットバターンで埋める::画面の初期化用
void oledFillData(uint8_t fillData) {
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト
  for (uint8_t m = 0; m < oledPages; m++) {
    // 表示開始位置を指定
    digitalWrite(OLED_DC, LOW); // Command mode
    SPI.transfer(0xB0 | (m & 0x07));
    SPI.transfer(0x10);
    SPI.transfer(0x00);

    // 表示データ転送
    // １バイトづつ書き込みだと折角のペリフェラルが勿体ないので
    // バッファを介して１セグメント分づつ送信している
    digitalWrite(OLED_DC, HIGH); // Data mode
    memset(SPIBuf,fillData,128) ;
    SPI.transfer(SPIBuf,128);
  }
  // 表示開始位置を指定
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(0xB0);
  SPI.transfer(0x10);
  SPI.transfer(0x00);

  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

// BITマップを表示する::oledFillDataの可変データ型なので基本oledFillDataと同じ作り
void oledBitMap(int offsetX , int offsetY , int width,int hight, const uint8_t bitMap[]) {
  int pos = 0 ;
  digitalWrite(OLED_CS, LOW); // SSD1306セレクト

  for (int m = 0; m < hight; m++) {
    // 表示開始位置を指定
    digitalWrite(OLED_DC, LOW); // Command mode
    SPI.transfer(0xB0 | ((m + offsetY) & 0x07));
    SPI.transfer(0x10 | (offsetX & 0xf0) >> 4 );
    SPI.transfer(offsetX & 0x0F);

    // 表示データ転送
    // Arduinoだとメモリが足りなくて１画面分を一気に転送する事ができない
    // なので、１セグメント分づつ送信している
    digitalWrite(OLED_DC, HIGH); // Data mode
    for (int x = 0; x < width; x++) {
      SPIBuf[x] = pgm_read_word(bitMap + pos);
      pos++ ;
    }
    SPI.transfer(SPIBuf,width);
  }
  // 表示開始位置を指定
  digitalWrite(OLED_DC, LOW); // Command mode
  SPI.transfer(0xB0);
  SPI.transfer(0x10);
  SPI.transfer(0x00);

  digitalWrite(OLED_CS, HIGH);  // SSD1306解放
}

void setup() {
  // --- SOFTWARE SPI ピン設定
  pinMode(OLED_CS, OUTPUT);   // Sleva Select
  pinMode(OLED_DC, OUTPUT);   // SSD1306 Set data/command pin as output

  //SPI初期化
  digitalWrite(OLED_CS, LOW);
  SPI.begin();//SPIを初期化、SCK、MOSI、SSの各ピンの動作は出力、SCK、MOSIはLOW、SSはHIGH
  SPI.setDataMode(SPI_MODE0);//SPIモードを「0」に設定・CPOL(クロック位相)=0,CPHA(クロック極性)=0
  SPI.setBitOrder(MSBFIRST);//SPI送受信用のビットオーダーを「MSBFIRST」に設定

  // --- Reset SSD1306
  pinMode(OLED_RESET, OUTPUT);// SSD1306 RESET
  digitalWrite(OLED_RESET, HIGH);
  delay(1);                       // VDD goes high at start, pause for 1 ms
  digitalWrite(OLED_RESET, LOW);  // Bring reset low
  delay(10);                      // Wait 10 ms
  digitalWrite(OLED_RESET, HIGH); // Bring out of reset

  // --- SSD1306 初期化
  // OLED設定コマンド送信
  oledSendCommands(sizeof(oledInitCommands), oledInitCommands) ;  // 初期化
  oledSendCommand(0xA4) ; // 描画ON
  oledSendCommand(0xAF) ; // 画面表示
}

void loop() {
  // ゆるぷろ
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(20 , 0 , 88 , 8, bitmapData); // Bitmap表示
  delay(2000) ;

  //　ささら
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(20 , 0 , 96 , 8, sasara); // Bitmap表示
  delay(2000) ;

  //　つづみ
  oledFillData(0xAA) ;    // 画面塗りつぶし
  oledBitMap(10 , 0 , 112 , 8, tudumi); // Bitmap表示
  delay(2000) ;
}
